###################################################################################
## R code for nonspatial null model applied to data sampled from 2-class population
## Murray Efford 2013-11-19
###################################################################################

CV <- function (x1, x2, f1, f2) {
    ## CV of variable with 2 values, frequencies f1 and f2
    sumx <- (f1*x1 + f2 *x2)
    mu <- sumx / (f1+f2)
    ((f1 * (x1-mu)^2 + f2 * (x2-mu)^2) / (f1+f2))^0.5 / mu
}
Nhat <- function (x, N, noccasion, ...) {
    ## random sample from 2-class population of size N
    p1 <- x[1]
    p2 <- x[2]
    pmix <- x[3]
    pop1 <- matrix(runif(N * noccasion * pmix) < p1, ncol = noccasion)
    pop2 <- matrix(runif(N * noccasion * (1-pmix)) < p2, ncol = noccasion)
    pop <- rbind(pop1,pop2)
    ## drop unobserved histories
    pop <- pop[apply(pop,1,sum)>0,]
    ## coerce to class 'capthist'
    class(pop) <- 'capthist'
    closedN(pop, ...)$Nhat
}
nonspatialsim <- function (nrepl = 10, N = 200, noccasion = 3,
                           p1 = seq(0.05, 0.40, 0.05),
                           p2 = seq(0.05, 0.40, 0.05),
                           pmix = 0.5) {
    ## need package secr for closedN()
    require(secr)
    parameters <- expand.grid(p1 = p1, p2 = p2, pmix = pmix)
    parameters$CV <- CV(parameters$p1, parameters$p2, 1,1)
    np <- nrow(parameters)
    runone <- function (x) {
        tmp <- apply(parameters, 1, Nhat, estimator = 'null', N = 200, noccasion = 3)
        tmp[abs(tmp) > (N*2)] <- NA
        tmp
    }
    out <- sapply(1:nrepl, runone)
    parameters$Nhat <- apply(out, 1, mean, na.rm = TRUE)
    parameters$validNhat <- apply(out, 1, function(x) sum(!is.na(x)))
    parameters$RBN <- (parameters$Nhat - N) / N
    parameters
}

## Run nonspatial bias simulation
nss3 <- nonspatialsim(nrepl = 1000, N = 200, noccasion = 3)
nss7 <- nonspatialsim(nrepl = 1000, N = 200, noccasion = 7)

## plot
xv <- seq(0,0.8,0.01); 
par(mfrow = c(1,2), mar = c(4,4,2,1), mgp = c(2.2,0.4,0), tck = 0.03, pty = 's',
    xaxs = 'r', yaxs = 'r', las=1)

plot(0,0, type = 'n', xlim = c(0, 0.8), ylim = c(-0.5,0.2),
     xlab = expression(CV(italic(p))), ylab = expression(RB(hat(italic(N)))))
points(nss3$CV, nss3$RBN, pch = 16,col = 'blue')
lines(xv, 1 / (1+ xv^2) - 1)
mtext(side = 3, line = 0, '3 occasions')

plot(0,0, type = 'n', xlim = c(0, 0.8), ylim = c(-0.5,0.2),
     xlab = expression(CV(italic(p))), ylab = expression(RB(hat(italic(N)))))
points(nss7$CV, nss7$RBN, pch = 16,col = 'blue')
lines(xv, 1 / (1+ xv^2) - 1)
mtext(side = 3, line = 0, '7 occasions')
###################################################################################

