###################################################################################
## R code for simulating SECR estimation from 2-class populations
## with varying lambda0 and sigma, using package 'secr'
## Murray Efford 2013-11-19
###################################################################################

## Step 1. Define function to run simulations

sim.h2 <- function (nrepl, trapset, maskset, parameters, skipfit = FALSE,
                    modelset = list(list(lambda0~1, sigma~1)), detectfn = 14,
                    distribution = c("poisson","binomial"), Ndist = c("poisson","fixed"),
                    method = "Newton-Raphson", model2D = 'poisson', extractfn,
                    ncores = 1, seed = 123, start = NULL) {
    ## Input arguments:
    ## nrepl        -- number of replicate simulations
    ## trapset      -- a list of traps objects, one for each layout
    ## maskset      -- a list of mask objects, one for each layout
    ## parameters   -- dataframe defining simulation scenarios, one per row
    ##                 required columns are :
    ##                   trapindex
    ##                   noccasions
    ##                   maskindex
    ##                   modelindex
    ##                   D.1G
    ##                   D.2G
    ##                   lambda0.1G
    ##                   lambda0.2G
    ##                   sigma.1G
    ##                   sigma.2G
    ## skipfit      -- logical; if TRUE data will be simulated but model fitting will be skipped
    ## distribution -- 'binomial' vs 'Poisson' distribution of N in
    ##                 masked area (see details$distribution in secr.fit)
    ## method       -- see argument of this name in secr.fit
    ## model2D      -- see argument of this name in sim.popn
    ## extractfn    -- function taking a fitted model as argument that returns a vector of values
    ## ncores       -- see ?Parallel in secr
    ## seed         -- random number seed used by clusterSetRNGStream (ncores>1) or set.seed
    ## start        -- starting values for coefficients on link scales

    ## Output: a list whose key component is 'output', a 3-dimensional
    ## array of numeric values containing the summarised output for each
    ## simulation. The first dimension of 'output' corresponds to
    ## scenarios (rows of 'parameters'), the second dimension corresponds
    ## to elements of the summary vector returned by extractfn(), and the
    ## third dimension to replicate. Other components of the list record
    ## the run time, input data, and extract function.

    #--------------------------------------------------------------------------

    sim.and.fit <- function (grid, nocc, mask, hcov, pNA = 0,
                             D.1, D.2, lambda0.1, lambda0.2, sigma.1, sigma.2) {

        pop1 <- sim.popn(D = D.1, core = attr(mask,"boundingbox"), buffer = 0, Ndist = Ndist)
        pop2 <- sim.popn(D = D.2, core = attr(mask,"boundingbox"), buffer = 0, Ndist = Ndist)
        if (pNA > 0) {
            n1 <- nrow(pop1); n2 <- nrow(pop2)
            covariates(pop1)$sex[sample.int (n1, size = n1 * pNA, replace = F)] <- NA
            covariates(pop2)$sex[sample.int (n2, size = n2 * pNA, replace = F)] <- NA
        }
        CH1 <- sim.capthist(grid, popn = pop1, detectpar = list(lambda0=lambda0.1,
            sigma = sigma.1), detectfn = detectfn, noccasions = nocc)
        CH2 <- sim.capthist(grid, popn = pop2, detectpar = list(lambda0=lambda0.2,
            sigma = sigma.2), detectfn = detectfn, noccasions = nocc)
        CH <- rbind.capthist(CH1, CH2, verify = FALSE)
        N <- nrow(pop1) + nrow(pop2)
        if (is.na(hcov)) hcov <- NULL
        fitmodel <- function(model) {
            if (skipfit | nrow(CH1)==0 | nrow(CH2)==0 | nrow(CH) < 10) {
                fit <- NA
            }
            else {
                fit <- try(secr.fit(CH, CL = TRUE, mask=mask, detectfn = detectfn,
                                    model = model, hcov =
                                    hcov, method = method, trace = FALSE,
                                    details = list(distribution =
                                    distribution), start = start) )
            }
            c(nrow(CH), sum(CH), N, extractfn(fit))
        }
        sapply(modelset, fitmodel)
    }
    #--------------------------------------------------------------------------
    runone <- function(x) {
        nt <- nrow(parameters)
        out <- array(dim=c(nt, 3+nval, nmod))   ## one replicate
        if (is.null(parameters$pNA))
            parameters$pNA <- rep(0,nt)
        for (tt in 1:nt) {
            out[tt,,] <- sim.and.fit (trapset[[parameters$trapindex[tt]]],
                                parameters$noccasions[tt],
                                maskset[[parameters$maskindex[tt]]],
                                hcov = parameters$hcov[tt],
                                pNA = parameters$pNA[tt],
                                D.1 = parameters$D.1G[tt],
                                D.2 = parameters$D.2G[tt],
                                lambda0.1 = parameters$lambda0.1G[tt],
                                lambda0.2 = parameters$lambda0.2G[tt],
                                sigma.1 = parameters$sigma.1G[tt],
                                sigma.2 = parameters$sigma.2G[tt]
                                     )
        }
        out
    }
    #--------------------------------------------------------------------------

    ## mainline of sim.h2()

    ## record start time etc.
    ptm  <- proc.time()
    cl   <- match.call(expand.dots = TRUE)
    starttime <- format(Sys.time(), "%H:%M:%S %d %b %Y")
    ## preprocess inputs
    if (inherits(trapset, 'traps'))   ## otherwise assume already list of traps
        trapset <- list(trapset)
    nk <- length(trapset)
    if (is.null(names(trapset)))
        names(trapset) <- paste('design',1:nk, sep='')
    nval <- length(extractfn(NA))     ## length of summary vector
    nmod <- length(modelset)
    if (is.null(names(modelset)))
        names(modelset) <- paste('model',1:nmod, sep='')
    ## run simulations
    if (ncores > 1) {
        clust <- makeCluster(ncores, methods = TRUE)
        clusterSetRNGStream(clust, seed)
        clusterEvalQ(clust, library(secr))
        clusterExport(clust, c("detectpar", "sim.and.fit", "modelset", "Ndist", "detectfn",
            "trapset", "maskset", "parameters","skipfit", "model2D", "start",
            "nval", "nmod", "distribution", "extractfn", "method"), environment())
        output <- parLapply(clust, 1:nrepl, runone)
        stopCluster(clust)
    }
    else {
        set.seed (seed)
        output <- lapply(1:nrepl, runone)
    }

    ## tidy up output
    output <- do.call(c, output)
    nt <- nrow(parameters)
    output <- array(output, dim = c(nt, 3+nval, nmod, nrepl),
                    dimnames = list (design = 1:nt,
                    c('n','ncapt', 'N', names(extractfn(NA))),
                    model = 1:nmod,
                    replicate = 1:nrepl))
    cat("Completed in", (proc.time() - ptm)[3], "seconds \n")
    value <- list (call = cl,
                   starttime = starttime,
                   trapset = trapset,
                   maskset = maskset,
                   extractfn = extractfn,
                   parameters = parameters,
                   proctime = (proc.time() - ptm)[3],
                   output = output
                   )
    value
}

###################################################################################
## Step 2. Define accessory functions

CV <- function (x1, x2, f1, f2) {
    ## compute CV of parameter x for 2-class population
    ## relative class frequencies f1, f2
    sumx <- (f1*x1 + f2 *x2)
    mu <- sumx / (f1+f2)
    ((f1 * (x1-mu)^2 + f2 * (x2-mu)^2) / (f1+f2))^0.5 / mu
}
addCV <- function (parameters, trapset, maskset, detectfn = 14) {
    ## add CV(lambda0), CV(sigma), CV(a) etc. to parameters dataframe
    ## assumes 50:50 ratio of class frequencies
    fx <- function(x) {
        tg <- trapset[[x[4]]]
        tm <- maskset[[x[5]]]
        pd <- pdot(tm, tg, detectfn = detectfn,
                   detectpar = list(lambda0 = x[1], sigma = x[2]), noccasions = x[3])
        sum(pd) * attr(tm, 'area')
    }
    tmp <- parameters
    tmp$CV.lambda0 <- CV(tmp$lambda0.1G, tmp$lambda0.2G, 1, 1)
    tmp$CV.sigma <- CV(tmp$sigma.1G, tmp$sigma.2G, 1, 1)
    tmp$CV.sigma2 <- CV(tmp$sigma.1G^2, tmp$sigma.2G^2, 1, 1)
    tmp$a.1 <- apply(tmp[,c('lambda0.1G','sigma.1G','noccasions','trapindex','maskindex')], 1, fx)
    tmp$a.2 <- apply(tmp[,c('lambda0.2G','sigma.2G','noccasions','trapindex','maskindex')], 1, fx)
    tmp$CV.a <- CV(tmp$a.1, tmp$a.2, 1, 1)
    tmp$CV.l0s2 <-  CV(tmp$lambda0.1G * tmp$sigma.1G^2, tmp$lambda0.2G*tmp$sigma.2G^2, 1, 1)
    fx1 <- function(x) {
        tg <- trapset[[x[4]]]
        tm <- maskset[[x[5]]]
        pd <- pdot(tm, tg, detectfn = detectfn,
                   detectpar = list(lambda0 = x[1], sigma = x[2]), noccasions = 1)
        sum(pd) * attr(tm, 'area')
    }
    a.1 <- apply(tmp[,c('lambda0.1G','sigma.1G','noccasions','trapindex','maskindex')], 1, fx1)
    a.2 <- apply(tmp[,c('lambda0.2G','sigma.2G','noccasions','trapindex','maskindex')], 1, fx1)
    tmp$CV.a1 <- CV(a.1, a.2, 1, 1)
    tmp
}

extractfn <- function(x) {
    ## function to extract desired summary of secr model
    if (inherits(x,'secr')) {
        unlist(derived(x, distribution="binomial")[2,])
    }
    else c(estimate=NA, SE.estimate=NA, lcl=NA, ucl=NA, CVn=NA, CVa=NA, CVD=NA)
}

###################################################################################
## Step 3. Construct detector layouts and habitat masks

library(secr)   ## load R package 'secr'
grid10x10 <- make.grid(nx = 10, ny = 10, spacing = 25, detector = 'proximity')
mask10x10 <- make.mask(grid10x10, type = 'traprect', buffer = 200, nx = 64)

###################################################################################
## Step 4. Build dataframe describing scenarios, one per row

## sim.h2() (above) allows for multiple traps, masks and models;
## we consider just one of each.
trapset <- list(grid10x10)
maskset <- list(mask10x10)
modelset <- list(fit0 = list(lambda0~1, sigma~1))

## Select a density that gives exactly N = 200 in bounding box
D <- 100 / (( (9*25)+400 )^2 *1e-4)

## Data frame to define generating (G) models
parameters <- expand.grid(noccasions = 3, trapindex = 1, maskindex = 1,
                          D.1G = D, D.2G = D, hcov=NA,
                          lambda0.1G = c(0.1,0.2,0.4), sigma.1G = c(1, 2^0.5, 2) * 25,
                          lambda0.2G = c(0.1,0.2,0.4), sigma.2G = c(1, 2^0.5, 2) * 25,
                          stringsAsFactors = FALSE
                          )

## Drop mirror-image scenarios (sex labels switched)
swap <- function(x) if (x[3]<x[1]) x[c(3,2,1,4)] else x
OK <- !duplicated(apply(apply(parameters[,6:9], 1, swap),2,paste, collapse=','))
parameters <- parameters[OK,]

## compute various CV and store in 'parameters' dataframe for later use
parameters <- addCV(parameters, trapset, maskset)

###################################################################################
## Step 5. Run all simulations

library(secr)

## for parallel execution on multi-core machine (ncores>1)
library(parallel)

simout <- sim.h2(nrepl = 2, trapset = trapset, maskset = maskset, modelset = modelset,
                 parameters = parameters, distribution = "Poisson", Ndist = "fixed",
                 method = "Newton-Raphson", extractfn = extractfn, ncores = 1,
                 seed = 567, skipfit = FALSE)

## see header of sim.h2 for structure of simout
###################################################################################
